/* =======================
 * リスト1
 * ======================= */

var canvas = document.getElementById("glcanvas");
var gl = canvas.getContext("experimental-webgl");

// 不透明度1.0の黒で画面をクリア
gl.clearColor(0.0, 0.0, 0.0, 1.0);

// 深度をクリア
gl.clearDepth(1.0);

// 深度バッファを計算する
gl.enable(gl.DEPTH_TEST);

// 遠くのものより近くのものを優先して描画
gl.depthFunc(gl.LEQUAL);


/* =======================
 * リスト2
 * ======================= */

// フラグメントシェーダーソース
var fragmentShaderSource = 'varying lowp vec4 vColor;\
                            void main(void) {\
                              gl_FragColor = vColor;\
                            }';

// 頂点シェーダーソース
var vertexShaderSource = 'attribute vec3 aVertexPosition;\
                          attribute vec4 aVertexColor;\
                          uniform mat4 uMVMatrix;\
                          uniform mat4 uPMatrix;\
                          varying lowp vec4 vColor;\
                          void main(void) {\
                            gl_Position = uPMatrix * uMVMatrix * vec4(aVertexPosition, 1.0);\
                            vColor = aVertexColor;\
                          }';

// シェーダーオブジェクトを作成
var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
var vertexShader = gl.createShader(gl.VERTEX_SHADER);

// シェーダーオブジェクトにソースコードを登録
gl.shaderSource(fragmentShader, fragmentShaderSource);
gl.shaderSource(vertexShader, vertexShaderSource);

// ソースコードからコンパイル
gl.compileShader(fragmentShader);
gl.compileShader(vertexShader);

// シェーダープログラムを作成とリンク
var shaderProgram = gl.createProgram();
gl.attachShader(shaderProgram, vertexShader);
gl.attachShader(shaderProgram, fragmentShader);
gl.linkProgram(shaderProgram);
gl.useProgram(shaderProgram);

// シェーダーの属性を登録
var vertexPositionAttribute = gl.getAttribLocation(shaderProgram, "aVertexPosition");
gl.enableVertexAttribArray(vertexPositionAttribute);
var vertexColorAttribute = gl.getAttribLocation(shaderProgram, "aVertexColor");
gl.enableVertexAttribArray(vertexColorAttribute);


/* =======================
 * リスト3
 * ======================= */

// 立方体の頂点バッファを作成
var cubeVerticesBuffer = gl.createBuffer();

// 頂点バッファとして登録
gl.bindBuffer(gl.ARRAY_BUFFER, cubeVerticesBuffer);

// 立方体の頂点を作成
var vertices = [
  -1, -1,  1,    1, -1,  1,    1,  1,  1,   -1,  1,  1, // 前面
  -1, -1, -1,   -1,  1, -1,    1,  1, -1,    1, -1, -1, // 背面
  -1,  1, -1,   -1,  1,  1,    1,  1,  1,    1,  1, -1, // 上面
  -1, -1, -1,    1, -1, -1,    1, -1,  1,   -1, -1,  1, // 下面
   1, -1, -1,    1,  1, -1,    1,  1,  1,    1, -1,  1, // 右面
  -1, -1, -1,   -1, -1,  1,   -1,  1,  1,   -1,  1, -1  // 左面
];

// 頂点の集合をFloat32Array形式に変換してからバッファに渡す
gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices), gl.STATIC_DRAW);

// 色の設定
var colors = [
  [1,  1,  1,  1], // 前面: 白
  [1,  0,  0,  1], // 背面: 赤
  [0,  1,  0,  1], // 上面: 緑
  [0,  0,  1,  1], // 下面: 青
  [1,  1,  0,  1], // 右面: 黄色
  [1,  0,  1,  1]  // 左面: 紫
];

// 色の配列を頂点のマッピング用に変換する
var generatedColors = [];

for (j = 0; j < 6; j++) {
  var c = colors[j];
  // 1面(4つの頂点)のために同じ色を4回繰り返す
  for (var i = 0; i < 4; i++) {
    generatedColors = generatedColors.concat(c);
  }
}

// 色バッファをWebGLコンテキストに登録
cubeVerticesColorBuffer = gl.createBuffer();
gl.bindBuffer(gl.ARRAY_BUFFER, cubeVerticesColorBuffer);
gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(generatedColors), gl.STATIC_DRAW);

// 要素バッファを作成
// これは立方体の各面の頂点のインデックスとなる
cubeVerticesIndexBuffer = gl.createBuffer();
gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, cubeVerticesIndexBuffer);

// この配列定義は各面で2つの三角形で構成される
// 各三角形の頂点が、頂点バッファでの座標を示すインデックスとなっている
var cubeVertexIndices = [
  0,  1,  2,    0,  2,  3,    // 前面
  4,  5,  6,    4,  6,  7,    // 背面
  8,  9,  10,   8,  10, 11,   // 上面
  12, 13, 14,   12, 14, 15,   // 下面
  16, 17, 18,   16, 18, 19,   // 右面
  20, 21, 22,   20, 22, 23    // 左面
]

// 要素バッファをWebGLコンテキストに登録
gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(cubeVertexIndices), gl.STATIC_DRAW);

/* =======================
 * リスト4
 * ======================= */

// canvasをクリアして描画を開始する
gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

// 遠近法のシーンを構築する
var perspectiveMatrix = makePerspective(45, 640.0/480.0, 0.1, 100.0);

// 描画位置をシーンの中央に設定する。
var mvMatrix = Matrix.I(4);

// 立方体の位置をZ軸方向に6.0移動させる
mvMatrix = Matrix.Translation($V([0.0, 0.0, -6.0])).ensure4x4();

// X軸に20度回転する
var matrix = Matrix.Rotation(20 * Math.PI / 180.0, $V([1, 0, 0])).ensure4x4();
mvMatrix = mvMatrix.x(matrix);

// Y軸に20度回転する
var matrix = Matrix.Rotation(30 * Math.PI / 180.0, $V([0, 1, 0])).ensure4x4();
mvMatrix = mvMatrix.x(matrix);

// バッファから立方体の頂点を取り出し、属性を設定する
gl.bindBuffer(gl.ARRAY_BUFFER, cubeVerticesBuffer);
gl.vertexAttribPointer(vertexPositionAttribute, 3, gl.FLOAT, false, 0, 0);

// 色情報の属性を設定する
gl.bindBuffer(gl.ARRAY_BUFFER, cubeVerticesColorBuffer);
gl.vertexAttribPointer(vertexColorAttribute, 4, gl.FLOAT, false, 0, 0);

// 立方体を描画する
gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, cubeVerticesIndexBuffer);
var pUniform = gl.getUniformLocation(shaderProgram, "uPMatrix");
gl.uniformMatrix4fv(pUniform, false, new Float32Array(perspectiveMatrix.flatten()));
var mvUniform = gl.getUniformLocation(shaderProgram, "uMVMatrix");
gl.uniformMatrix4fv(mvUniform, false, new Float32Array(mvMatrix.flatten()));
gl.drawElements(gl.TRIANGLES, 36, gl.UNSIGNED_SHORT, 0);

